english_word_dict = {
    "main_window_current_filename_format": "Current Filename: {name}",
    # File menu
    "main_window_open_image": "Open File",
    "main_window_current_filename": "Current Filename:",
    "main_window_current_file": "File",
    "main_window_open_folder": "Open Folder",
    "main_window_exit": "Exit",
    "main_window_tile_size": "Thumbnail Size",
    "main_window_select_folder": "Select Folder",
    "main_window_current_folder_format": "Current Folder: {path}",
    "main_window_remove_undo_stack": "Remove temp undo files",
    # Language Menubar
    "menu_bar_language": "Language",
    "language_menu_bar_please_restart_messagebox": "Please restart",
    "language_menu_bar_english": "English",
    "language_menu_bar_traditional_chinese": "Traditional Chinese",
    "language_menu_bar_chinese": "Chinese",
    "language_menu_bar_koren": "Korean",
    "language_menu_bar_japanese": "Japanese",
    # Tip menu
    "main_window_tip_menu": "Instructions",
    "main_window_mouse_tip_menu": "Mouse Control",
    "main_window_keyboard_tip_menu": "Keyboard Control",
    "mouse_control_middle_tip": "You can move using the mouse middle button",
    "mouse_control_left_tip": "Clicking an image with the left mouse button enters Deep zoom mode",
    "mouse_control_right_move_tip": "Hold down the right mouse button to multi-select images in thumbnail mode",
    "keyboard_control_esc_tip": "Press ESC to exit Deep zoom mode",
    "keyboard_control_tile_arrow_up_tip": "In thumbnail mode, use the Up arrow key to move upward",
    "keyboard_control_tile_arrow_down_tip": "In thumbnail mode, use the Down arrow key to move downward",
    "keyboard_control_tile_arrow_left_tip": "In thumbnail mode, use the Left arrow key to move left",
    "keyboard_control_tile_arrow_right_tip": "In thumbnail mode, use the Right arrow key to move right",
    "keyboard_control_delete_tip": "You can use Delete to remove images in both thumbnail and Deep zoom modes",
    "keyboard_r_tip": "Press the R key on the keyboard to reset the coordinates",
    # Right click menu
    "right_click_menu_go_to_parent_folder": "Go to Parent Folder",
    "right_click_menu_next_image": "Next Image",
    "right_click_menu_previous_image": "Previous Image",
    "right_click_menu_delete_current": "Delete Current Image",
    "right_click_menu_delete_selected": "Delete Selected Images",
    "right_click_menu_image_info": "Image Info",
    # Image info
    "image_info_filename": "Filename: {info}\n",
    "image_info_fullpath": "Full Path: {full_path}\n",
    "image_info_image_size": "Size: {width} x {height}\n",
    "image_info_file_size": "File Size: {file_size_mb} MB\n",
    "image_info_file_created_time": "Created Time: {created_time}\n",
    "image_info_file_modified_time": "Modified Time: {modified_time}\n",
    "image_info_messagebox_title": "Image Info",
    # Image info exif
    "image_info_exif_datatime_original": "Taken Time: {DateTimeOriginal}\n",
    "image_info_exif_camera_model": "Camera: {Make} {Model}\n",
    "image_info_exif_camera_lens_model": "Lens: {LensModel}\n",
    "image_info_exif_camera_focal_length": "Focal Length: {FocalLength}\n",
    "image_info_exif_camera_fnumber": "Aperture: {FNumber}\n",
    "image_info_exif_exposure_time": "Shutter: {ExposureTime}\n",
    "image_info_exif_iso": "ISO: {ISOSpeedRatings}",
    # Recent menu
    "recent_menu_title": "Recently Opened Menu",
}


traditional_chinese_word_dict = {
    "main_window_current_filename_format": "目前檔名： {name}",
    # File menu
    "main_window_open_image": "開啟檔案",
    "main_window_current_filename": "目前檔名：",
    "main_window_current_file": "檔案",
    "main_window_open_folder": "開啟資料夾",
    "main_window_exit": "離開",
    "main_window_tile_size": "縮圖大小",
    "main_window_select_folder": "選擇資料夾",
    "main_window_current_folder_format": "目前資料夾： {path}",
    "main_window_remove_undo_stack": "刪除暫存撤銷檔案",
    # Language Menubar
    "menu_bar_language": "語言",
    "language_menu_bar_please_restart_messagebox": "請重新啟動",
    "language_menu_bar_english": "英文",
    "language_menu_bar_traditional_chinese": "繁體中文",
    "language_menu_bar_chinese": "簡體中文",
    "language_menu_bar_koren": "韓文",
    "language_menu_bar_japanese": "日文",
    # Tip menu
    "main_window_tip_menu": "使用說明",
    "main_window_mouse_tip_menu": "滑鼠操作",
    "main_window_keyboard_tip_menu": "鍵盤操作",
    "mouse_control_middle_tip": "可以使用滑鼠中鍵移動",
    "mouse_control_left_tip": "滑鼠左鍵點擊圖片可以進入 Deep zoom 模式",
    "mouse_control_right_move_tip": "滑鼠右鍵按住可以在縮圖模式下複選圖片",
    "keyboard_control_esc_tip": "Deep zoom 模式下可以按 ESC 退出",
    "keyboard_control_tile_arrow_up_tip": "縮圖模式下可以使用方向鍵的上來往上移動",
    "keyboard_control_tile_arrow_down_tip": "縮圖模式下可以使用方向鍵的下來往下移動",
    "keyboard_control_tile_arrow_left_tip": "縮圖模式下可以使用方向鍵的左來往左移動",
    "keyboard_control_tile_arrow_right_tip": "縮圖模式下可以使用方向鍵的右來往右移動",
    "keyboard_control_delete_tip": "縮圖或 Deep zoom 模式都可以使用 Delete 來刪除圖片",
    "keyboard_r_tip": "可以按鍵盤上的 R 鍵，來重置座標",
    # Right click menu
    "right_click_menu_go_to_parent_folder": "前往父資料夾",
    "right_click_menu_next_image": "下一張圖",
    "right_click_menu_previous_image": "上一張圖",
    "right_click_menu_delete_current": "刪除目前的圖",
    "right_click_menu_delete_selected": "刪除選取的圖",
    "right_click_menu_image_info": "圖片資訊",
    # Image info
    "image_info_filename": "檔名: {info}\n",
    "image_info_fullpath": "完整路徑: {full_path}\n",
    "image_info_image_size": "尺寸: {width} x {height}\n",
    "image_info_file_size": "檔案大小: {file_size_mb} MB\n",
    "image_info_file_created_time": "建立時間: {created_time}\n",
    "image_info_file_modified_time": "修改時間: {modified_time}\n",
    "image_info_messagebox_title": "圖片資訊",
    # Image info exif
    "image_info_exif_datatime_original": "拍攝時間: {DateTimeOriginal}\n",
    "image_info_exif_camera_model": "相機: {Make} {Model}\n",
    "image_info_exif_camera_lens_model": "鏡頭: {LensModel}\n",
    "image_info_exif_camera_focal_length": "焦段: {FocalLength}\n",
    "image_info_exif_camera_fnumber": "光圈: {'FNumber'}\n",
    "image_info_exif_exposure_time": "快門: {ExposureTime}\n",
    "image_info_exif_iso": "ISO: {ISOSpeedRatings}",
    # Recent menu
    "recent_menu_title": "近期開啟選單",
}


from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from PySide6.QtGui import QActionGroup
from PySide6.QtWidgets import QFileDialog

from Imervue.gpu_image_view.actions.delete import commit_pending_deletions
from Imervue.gpu_image_view.images.image_loader import load_image
from Imervue.menu.recent_menu import rebuild_recent_menu, build_recent_menu
from Imervue.user_settings.recent_image import add_to_recent, add_recent_folder, add_recent_image
from Imervue.user_settings.user_setting_dict import user_setting_dict

if TYPE_CHECKING:
    from Imervue.Imervue_main_window import ImervueMainWindow

from Imervue.multi_language.language_wrapper import language_wrapper


def build_file_menu(ui_we_want_to_set: ImervueMainWindow):
    # ===== 檔案 =====
    file_menu = ui_we_want_to_set.menuBar().addMenu(language_wrapper.language_word_dict.get("main_window_current_file"))

    open_image_action = file_menu.addAction(
        language_wrapper.language_word_dict.get("main_window_open_image")
    )
    open_image_action.triggered.connect(
        lambda: open_image(ui_we_want_to_set)
    )

    open_folder_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_open_folder"))
    open_folder_action.triggered.connect(lambda: open_folder(ui_we_want_to_set))

    build_recent_menu(ui_we_want_to_set=ui_we_want_to_set, menu=file_menu)

    # 新增刪除動作
    delete_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_remove_undo_stack"))
    delete_action.triggered.connect(lambda: commit_pending_deletions(ui_we_want_to_set.viewer))

    exit_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_exit"))
    exit_action.triggered.connect(ui_we_want_to_set.close)

    # ===== Tile Size 選單 =====
    view_menu = ui_we_want_to_set.menuBar().addMenu(language_wrapper.language_word_dict.get("main_window_tile_size"))

    tile_group = QActionGroup(ui_we_want_to_set)
    tile_group.setExclusive(True)

    thumbnail_size = [128, 256, 512, 1024, "None"]

    for size in thumbnail_size:
        if size != "None":
            action = view_menu.addAction(f"{size} x {size}")
        else:
            action = view_menu.addAction(size)
        action.setCheckable(True)

        if size == ui_we_want_to_set.viewer.thumbnail_size:
            action.setChecked(True)

        tile_group.addAction(action)

        action.triggered.connect(
            lambda checked, s=size: ui_we_want_to_set.change_tile_size(s)
        )
    return file_menu


# ==========================
# 開啟資料夾
# ==========================
def open_folder(ui_we_want_to_set: ImervueMainWindow):
    folder = QFileDialog.getExistingDirectory(
        ui_we_want_to_set, language_wrapper.language_word_dict.get("main_window_select_folder"))

    if folder:
        ui_we_want_to_set.tree.setRootIndex(ui_we_want_to_set.model.index(folder))
        add_recent_folder(folder)
        rebuild_recent_menu(ui_we_want_to_set)
        user_setting_dict["user_last_folder"] = folder

# ==========================
# 開啟單張圖片
# ==========================
def open_image(ui_we_want_to_set: ImervueMainWindow):
    file_path, _ = QFileDialog.getOpenFileName(
        ui_we_want_to_set,
        language_wrapper.language_word_dict.get("main_window_select_image"),
        "",
        "Images (*.png *.jpg *.jpeg *.bmp *.tiff)"
    )

    if not file_path:
        return

    # 清除 tile grid 狀態
    ui_we_want_to_set.viewer.clear_tile_grid()

    # 呼叫你提供的函式
    load_image(file_path, ui_we_want_to_set.viewer)

    # 設定 viewer 狀態
    ui_we_want_to_set.viewer.tile_grid_mode = False
    ui_we_want_to_set.viewer.current_index = 0
    ui_we_want_to_set.viewer.model.set_images([file_path])

    # 更新檔名 label
    ui_we_want_to_set.filename_label.setText(
        language_wrapper.language_word_dict.get(
            "main_window_current_filename_format"
        ).format(name=Path(file_path).name))

    add_recent_image(file_path)
    rebuild_recent_menu(ui_we_want_to_set)

